# -*- coding: utf-8 -*-
"""Test the TcEx Batch Module."""
# import time

# import pytest

# from ..tcex_init import tcex


# pylint: disable=R0201,W0201
class TestUtils:
    """Test the TcEx Batch Module."""

    def setup_class(self):
        """Configure setup before all tests."""
        # self.tcex = test()

    # def test_token_renewal(self):
    #     """Test token renewal"""
    #     expiration_time = int(time.time())
    #     owner_resource._request.session.auth._token_expiration = int(time.time())
